
# cmm_comp.py

from __future__ import print_function

from ckit_parser import *

import copy

# --------------------------------------------------------------------------

class Compiler (Parser) :
   def __init__ (self) :
       super (Compiler, self).__init__ ()

       self.global_scope = CmmNamespaceDecl ()
       self.global_scope.item_name = ""
       self.global_scope.qual_name = ""
       self.global_scope.item_dict = { }
       self.global_scope.item_list = [ ]
       self.display = [ self.global_scope ]

   # scope

   def enter (self, item, also_mark_defn = True) :
       top = self.display [-1]
       top.item_dict [item.item_name] = item
       top.item_list.append (item)
       item.item_context = top
       if top.qual_name == "" :
          item.qual_name = item.item_name
       else :
          item.qual_name = top.qual_name + "." + item.item_name
       if also_mark_defn :
          self.markDefn (item)

   def openScope (self, item, already_entered = False) :
       if not already_entered :
          self.enter (item)
       if not hasattr (item, "item_dict") :
          item.item_dict = { }
       if not hasattr (item, "item_list") :
          item.item_list = [ ]
       self.display.append (item)

   def closeScope (self) :
       self.display.pop ()

   def lookup (self, name) :
       result = None
       inx = len (self.display) - 1
       while result == None and inx >= 0 :
          result = self.display [inx].item_dict.get (name)
          inx = inx - 1
       return result

   def getName (self, name) :
       return name.id

   # namespace

   def open_namespace (self, module) :
       module.item_name = self.getName (module.name)
       module.item_expand = True
       module.icon = "module"
       self.openScope (module)
       self.openCompletion (module)
       self.markOutline (module)

   def close_module (self, module) :
       self.closeCompletion (module)
       self.closeScope ()

   # class

   def open_class (self, cls) :
       cls.item_name = self.getName (cls.name)
       cls.item_expand = True
       cls.icon = "class"
       self.openScope (cls) # scope for identifiers
       self.openCompletion (cls) # area with text completion
       self.openRegion (cls) # region with background color
       self.markOutline (cls) # navigator item

   def close_class (self, cls) :
       self.closeRegion ()
       self.closeCompletion (cls)
       self.closeScope ()

   # enum

   def open_enum (self, enum) :
       enum.item_name = self.getName (enum.name)
       enum.icon = "enum"
       self.openScope (enum)
       self.openCompletion (enum)
       self.openRegion (enum)
       self.markOutline (enum)

   def on_enum_item (self, item) :
       item.item_name = self.getName (item.name)
       item.icon = "variable"
       self.enter (item)

   def close_enum (self, enum) :
       self.closeCompletion (enum)
       self.closeRegion ()
       self.closeScope ()

   # variable

   def add_init_declarator (self, decl) :
       if decl.name != None :
          decl.item_name = self.getName (decl.name)
          decl.icon = "variable"
          self.enter (decl)

   # fuction

   #def open_function (self, simple_declaration) :
       #simple_item = simple_declaration.items [-1]
       #simple_item.icon = "function"
       #self.openRegion (simple_item)
       #self.openScope (simple_item, already_entered = True)
       #self.openCompletion (simple_item)
       #self.markOutline (simple_item)

   #def close_function (self, simple_declaration) :
       #simple_item = simple_declaration.items [-1]
       #self.closeCompletion (simple_item)
       #self.closeScope ()
       #self.closeRegion ()
       #simple_declaration.func_body.item_label = simple_item.item_name

   def is_parameter (self) :
       return True

# --------------------------------------------------------------------------

# kate: indent-width 1; show-tabs true; replace-tabs true; remove-trailing-spaces all
